/*  cropdm.c -- program to crop a DM file  */


#include <stdlib.h>
#include <stdio.h>
#include "dmops.h"

static void usage(char *progname);



main(int argc, char *argv[])
{
    FILE *fp;
    DMdata *dm;
    int x0, y0, x1, y1, y;
    UV swUV, nwUV, neUV, seUV;
    UV swXY, nwXY, neXY, seXY;
    Transform *f;

    if (argc < 2) {
	usage(argv[0]);
    }

    fp = fopen(argv[1], "rb");
    if (fp == NULL) {
	fprintf(stderr, "Can't open %s for read\n", argv[1]);
	exit(1);
    }
    dm = dmRead(fp);

    x0 = 0;
    y0 = 0;
    x1 = dm->width;
    y1 = dm->height;

    switch(argc) {
    case 2:
	break;
    case 4:
	x1 = atoi(argv[2]);
	y1 = atoi(argv[3]);
	break;
    case 6:
	x0 = atoi(argv[2]);
	y0 = atoi(argv[3]);
	x1 = atoi(argv[4]);
	y1 = atoi(argv[5]);
	break;
    default:
	usage(argv[0]);
	break;
    }

    if (x0<0 || y0<0 || x1>=dm->width || y1>=dm->height || x0 > x1 || y0 > y1) {
	fprintf(stderr, "Invalid window size for %d x %d dm file.\n",
			dm->width, dm->height);
	exit(1);
    }

    f = createTransform(dm->swXY, dm->nwXY, dm->neXY, dm->seXY,
			dm->swUV, dm->nwUV, dm->neUV, dm->seUV);

    /* transform given indices to find uv coords of selected region */
    swXY.u = x0;
    swXY.v = y0;
    nwXY.u = x0;
    nwXY.v = y1;
    neXY.u = x1;
    neXY.v = y1;
    seXY.u = x1;
    seXY.v = y0;

    transform(f, swXY, &swUV);
    transform(f, nwXY, &nwUV);
    transform(f, neXY, &neUV);
    transform(f, seXY, &seUV);

    /* the new uv coords have the following new row, column indices */
    swXY.u = 0;
    swXY.v = 0;
    nwXY.u = 0;
    nwXY.v = y1 - y0 + 1;
    neXY.u = x1 - x0 + 1;
    neXY.v = y1 - y0 + 1;
    seXY.u = x1 - x0 + 1;
    seXY.v = 0;

    dmWriteHeader(stdout, x1 - x0 + 1, y1 - y0 + 1,
		swUV, nwUV, neUV, seUV,
		swXY, nwXY, neXY, seXY);

    for (y = y0; y <= y1; y++) {
	fwrite(&(dm->data[y*dm->width+x0]), sizeof(short), x1 - x0 + 1, stdout);
    }

    return 0;
}



static void usage(char *progname)
{
    fprintf(stderr,
	"usage:  %s filename [[x0 y0] x1 y1]\n", progname);
    fprintf(stderr,
	"\tx0 y0\tSouth-West corner (default 0 0)\n");
    fprintf(stderr,
"\tx1 y1\tNorth-East corner (default N-E corner of original)\n");
    exit(1);
}
