/*  defns.h --  structures, types, and macros for TopoVista  */


/*
 * Hierarchy Node definition
 */

#define MAXERR 65535		/* maximum scaled triangle error */

/* Node info flag bits */
#define SMALLER1	  1	/* Is neighbor 1 smaller? */
#define SMALLER2	  2	/* Is neighbor 2 smaller? */
#define BIGGER3		  4	/* Is neighbor 3 larger? */
#define SURFACE		  8	/* on surface? */
#define ALLINFOV	 16	/* completely in field of view? */
#define PARTINFOV	 32	/* partially in field of view? */
#define ALLINROD	 64	/* completely in rectangle of definition? */
#define PARTINROD	128	/* partially in rectangle of definition? */

typedef struct {		/* one node of triangle hierarchy */
    unsigned short err;		/* error, 0 to MAXERR */
    char info;			/* flags */
} Node;

/* info field access macros */
#define BORDER(hp, t) ((hp)->node[t].info & PARTINROD)
#define INROD(hp, t) ((hp)->node[t].info & ALLINROD)
#define PUTONSURFACE(hp, t) \
    (hp)->node[t].info = (hp)->node[t].info & (ALLINROD | PARTINROD) | SURFACE
#define ONSURFACE(hp, t) ((hp)->node[t].info & SURFACE)
#define INFOV(hp, t) ((hp)->node[t].info & (ALLINFOV | PARTINFOV))
#define ISLEAF(hp, t) ((t)<<1 >= (hp)->size-1)
#define SMALLN1(hp, t) ((hp)->node[t].info & SMALLER1)
#define SMALLN2(hp, t) ((hp)->node[t].info & SMALLER2)
#define BIGN3(hp, t) ((hp)->node[t].info & BIGGER3)



/*
 * Heirarchy definition
 */
typedef struct {		/* triangle hierarchy */
    int depth;			/* maximum depth */
    int size;			/* size */
    int numSurfTri;		/* number of triangles */
    Node *node;			/* top node */
} Hier;


typedef int TriIndex;
typedef int Coord2d[2];


typedef struct {		/* rgb spec for elevation coloring */
    float red;
    float green;
    float blue;
} Color;

#define MAXBAD     65535	/* maximum scaled triangle "badness" */

#define YCOORDMASK 0x000007FF
#define XCOORDMASK 0x003FF800

#define LCHILD(i) (2*(i)+1)	/* index of left child, given the parent */
#define RCHILD(i) (2*(i)+2)	/* index of right child, given the parent */
#define PARENT(i) (((i)-1)/2)	/* index of parent, given the child */



/* terrain display modes */
#define T_CONTOURS 0	/* color-coded elevation contours */
#define T_GRAY 1	/* grayscale relief only */
#define T_RGB 2		/* input-specified colors */

/* 3-D display modes */
#define HEAP 0		/* heap mode display */
#define ERR 1		/* error threshold display */
#define DIST 2		/* distance threshold (with error, too) */

/* surface generation modes */
#define NONE2D	0	/* flag for no 2D mesh */
#define WIRE2D	1
#define FILL2D	2
#define WIRE3D	3
#define FILL3D	4
