/*  dem2dm.c -- convert DEM file to DM file for use by TopoVista  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "dmops.h"
#include "dem.h"

main(int argc, char *argv[])
{
    FILE *fp;
    Dem *dp;
    int i, j;
    int zone;
    UV swXY, nwXY, neXY, seXY;
    UV swUV, nwUV, neUV, seUV;
    UV min;

    if (argc > 2) {
	fprintf(stderr, "Usage: %s [filename]\n", argv[0]);
	exit(1);
    }

    if (argc < 2)
	fp = stdin;
    else {
	fp = fopen(argv[1], "r");
	if (fp == NULL) {
	    fprintf(stderr, "Can't open %s for read\n", argv[1]);
	    exit(1);
	}
    }

    dp = malloc(sizeof(Dem));
    if (dp == NULL) {
	fprintf(stderr, "Not enough memory to hold DEM.\n");
	exit(1);
    }


    readDEM(fp, dp);

    fprintf(stderr, "%s\n", dp->name);
    fprintf(stderr, "Writing dm file...\n");

    for (i = 0; i < 4; i++) {
	if (i == 0) {
	    min.u = dp->corner[i].u;
	    min.v = dp->corner[i].v;
	} else {
	    min.u = MIN(min.u, dp->corner[i].u);
	    min.v = MIN(min.v, dp->corner[i].v);
	}
    }
    swXY.u = floor((dp->corner[0].u - min.u) / dp->resolution[0]);
    if (swXY.u < 0.0)
	swXY.u = 0.0;
    swXY.v = floor((dp->corner[0].v - min.v) / dp->resolution[1]);
    if (swXY.v < 0.0)
	swXY.v = 0.0;
    nwXY.u = floor((dp->corner[1].u - min.u) / dp->resolution[0]);
    if (nwXY.u < 0.0)
	nwXY.u = 0.0;
    nwXY.v = floor((dp->corner[1].v - min.v) / dp->resolution[1]);
    if (nwXY.v > dp->rows - 1)
	nwXY.v = dp->rows - 1;
    neXY.u = floor((dp->corner[2].u - min.u) / dp->resolution[0]);
    if (neXY.u > dp->cols - 1)
	neXY.u = dp->cols - 1;
    neXY.v = floor((dp->corner[2].v - min.v) / dp->resolution[1]);
    if (neXY.v > dp->rows - 1)
	neXY.v = dp->rows - 1;
    seXY.u = floor((dp->corner[3].u - min.u) / dp->resolution[0]);
    if (seXY.u > dp->cols - 1)
	seXY.u = dp->cols - 1;
    seXY.v = floor((dp->corner[3].v - min.v) / dp->resolution[1]);
    if (seXY.v < 0.0)
	seXY.v = 0.0;

    if (dp->reference_system == 1 && dp->ground_unit == 2) {

	/* coordinate system in UTM with meter ground units */

	utm2ll(dp->zone, dp->corner[0], &swUV);
	utm2ll(dp->zone, dp->corner[1], &nwUV);
	utm2ll(dp->zone, dp->corner[2], &neUV);
	utm2ll(dp->zone, dp->corner[3], &seUV);

    } else {

	/* Hope the coordinate system is in arcseconds */

	swUV.u = dp->corner[0].u;
	swUV.v = dp->corner[0].v;
	nwUV.u = dp->corner[1].u;
	nwUV.v = dp->corner[1].v;
	neUV.u = dp->corner[2].u;
	neUV.v = dp->corner[2].v;
	seUV.u = dp->corner[3].u;
	seUV.v = dp->corner[3].v;
    }

    dmWriteHeader(stdout, dp->cols, dp->rows,
		swUV, nwUV, neUV, seUV,
		swXY, nwXY, neXY, seXY);

    for(j = 0; j < dp->rows; ++j) {
	for(i = 0; i < dp->cols; ++i) {
	    fwrite(&(dp->data[i][j]), sizeof(unsigned short), 1, stdout);
	}
    }

    return 0;
}
