/*  dmops.c -- operations on DM (packed DEM) files  */

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include "dmops.h"



#define MAGIC_VALUE 0x04030201

typedef union {
	unsigned int val;
	unsigned char bytes[4];
} bitTest;



static int dmMatchOrder(char orderBytes[4])
{
	int match = 1;
	int i;
	bitTest test;

	test.val = MAGIC_VALUE;

	for (i = 0; i < 3; i++)
		match = match & (test.bytes[i] == orderBytes[i]);

	return match;
}



static void dmSwapOrder(short *data, int length)
{
	int i;

	for(i = 0; i < length; i++)
		data[i] = (data[i] << 8) & 0xff00 | (data[i] >> 8) & 0xff;
}



void dmReadHeader(FILE *in, DMdata *dm, int *shouldSwap)
{
	int n;
	char orderBytes[4];

	n = fscanf(in, "DM %d %d %c%c%c%c\n",
			&dm->width, &dm->height,
		        &orderBytes[0],
		        &orderBytes[1],
		        &orderBytes[2],
		        &orderBytes[3]);
	if (n == EOF) {
	    fprintf(stderr, "Empty input file.\n");
	    exit(1);
	}
	if (n == 0) {
	    fprintf(stderr, "Not a DM file.\n");
	    exit(1);
	}

	fscanf(in, "%lf %lf %lf %lf %lf %lf %lf %lf\n",
		    &(dm->swUV.u), &(dm->swUV.v),
		    &(dm->nwUV.u), &(dm->nwUV.v),
		    &(dm->neUV.u), &(dm->neUV.v),
		    &(dm->seUV.u), &(dm->seUV.v));
	fscanf(in, "%lf %lf %lf %lf %lf %lf %lf %lf\n",
		    &(dm->swXY.u), &(dm->swXY.v),
		    &(dm->nwXY.u), &(dm->nwXY.v),
		    &(dm->neXY.u), &(dm->neXY.v),
		    &(dm->seXY.u), &(dm->seXY.v));

	*shouldSwap = !dmMatchOrder(orderBytes);
}



void dmWriteHeader(FILE *out, int width, int height,
		    UV swUV, UV nwUV, UV neUV, UV seUV,
		    UV swXY, UV nwXY, UV neXY, UV seXY)
{
	bitTest test;

	test.val = MAGIC_VALUE;
	fprintf(out, "DM %d %d %c%c%c%c\n",
			width, height,
			test.bytes[0],
			test.bytes[1],
			test.bytes[2],
			test.bytes[3]);
	fprintf(out, "%lf %lf %lf %lf %lf %lf %lf %lf\n",
			swUV.u, swUV.v,
			nwUV.u, nwUV.v,
			neUV.u, neUV.v,
			seUV.u, seUV.v);
	fprintf(out, "%lf %lf %lf %lf %lf %lf %lf %lf\n",
			swXY.u, swXY.v,
			nwXY.u, nwXY.v,
			neXY.u, neXY.v,
			seXY.u, seXY.v);
}



DMdata *dmRead(FILE *in)
{
	DMdata *dm;
	int shouldSwap;
	int length;

	dm = malloc(sizeof(DMdata));
	if (!dm) {
		fprintf(stderr, "dmRead: Unable to allocate memory.\n");
		exit(1);
	}

	dmReadHeader(in, dm, &shouldSwap);

	length = dm->width * dm->height;

	dm->data = malloc(sizeof(short) * length);
	if (!dm->data) {
		fprintf(stderr, "dmRead: Unable to allocate memory.\n");
		exit(1);
	}
	fread(dm->data, sizeof(short), length, in);

	if (shouldSwap)
		dmSwapOrder(dm->data, length);
	return dm;
}



void dmWrite(FILE *out, DMdata *dm)
{
    dmWriteHeader(out, dm->width, dm->height,
		dm->swUV, dm->nwUV, dm->neUV, dm->seUV,
		dm->swXY, dm->nwXY, dm->neXY, dm->seXY);

    fwrite(dm->data, sizeof(short), dm->width * dm->height, out);
}
