/*  globals.h -- global variables of TopoVista  */


/*
 *  variables prefaced by "global" are both declared and defined by this
 *  file, depending on how that macro is defined.
 */

#ifndef global
#define global extern
#define initial(v) /**/
#endif



/* windows (GL/GLUT handles) */

global int ctlWin;		/* control window */
global int ovhWin;		/* overhead ("2-D") window */
global int pspWin;		/* perspective ("3-D" window */



/* terrain data */

global RGBdata *cgrid;		/* RGB data grid */

global char *dmname;		/* name of .dm file */
global DMdata *grid;		/* DEM data grid */

global int XMAX, YMAX;		/* maximum x, y indices in grid data */
global int DIMENSION;		/* hier. side (power of 2 >= max(XMAX, YMAX)) */

global float xmeters, ymeters;	/* dimensions of data in meters */

global int EMIN, EMAX;		/* min/max elevation in raw data */
global float ZMIN, ZMAX;	/* min/max after scaling to arc-second units */

global float errScale;		/* scales +-MAXERR to true error value */

global Hier *hierarchy;		/* hierarchy pointer */



/*  viewing options  */

global int surfMethod	initial(DIST);	/* surface construction method */
global int reliefMethod;		/* relief coloring method */

global int twoDMesh	initial(NONE2D); /* show 2-D mesh? */
global int threeDMesh	initial(0);	/* show 3-D mesh? */

global int eyeUp	initial(1);	/* is eye above data plane? */



/*  rendering parameters  */

global float errThresh          initial(0.0002);	/* error threshold */
global float errThreshInc       initial(0.00001);	/* increment */

global int heapThresh		initial(6000);		/* heap threshold */
global int heapThreshInc	initial(2000);		/* increment */

global double stepThresh	initial(-5.0);		/* distance threshold */
global double stepThreshInc	initial(0.5);		/* increment */

global float vscale;					/* vertical scaling */



/* current viewpoint */

global float eyeWx, eyeWy, eyeWz;	/* eye location */
global float refWx, refWy, refWz;	/* reference point */
global float Rx, Ry, Rz;		/* reference vector */



/* other */

global float inAngleThresh;

global Coord2d se, sw, ne, nw;



/* defined in main.c (too complex for macros above): */

extern Color elevMap[];		/* elevation colors */
extern int elevCount;		/* number of elevation colors */



/* these macros should not be used elsewhere */
#undef global
#undef initial
