/*  heap.c -- heap operations  */

/* by Will Evans 9/20/96 (adapted from Garland and Heckbert) */


#include "topo.h"

#define MAXHEAPSIZE 100000

#define HEAP_PARENT(i)	(((i)-1)/2)
#define HEAP_LEFT(i)	(2*(i)+1)
#define HEAP_RIGHT(i)	(2*(i)+2)

typedef struct {
    TriIndex t;
    int val;
} HeapNode;

static int heapSize = 0;
static HeapNode heap[MAXHEAPSIZE];



/*  printHeap() -- print the heap for debugging purposes  */

static void printHeap(void)
{
    int row = 1;
    int i;

    for(i = 0; i < heapSize; ++i) {
	printf("%d (%3.0f)  ", heap[i].t, heap[i].val);
	if (i == row - 1) {
	    row = 2 * row + 1;
	    printf("\n");
	}
    }
    printf("\n");
}



/*  heapSwap(i, j) -- interchange two heap items  */

static void heapSwap(int i, int j)
{
    HeapNode tmp = heap[i];

    heap[i] = heap[j];
    heap[j] = tmp;
}


/*
 *  upheap(i) -- move node i up in the heap, if necessary
 *  downheap(i) -- move node i down through the heap, if necessary
 *
 *  (note: both of these perform more swapping that is strictly necessary)
 */

static void upheap(int i)
{
    if (i == 0)
	return;

    if (heap[i].val > heap[HEAP_PARENT(i)].val) {
	heapSwap(i, HEAP_PARENT(i));
	upheap(HEAP_PARENT(i));
    }
}

static void downheap(int i)
{
    int largest, l, r;

    if (i >= heapSize)
	return;			/* perhaps just extracted the last */

    largest = i;
    l = HEAP_LEFT(i);
    r = HEAP_RIGHT(i);

    if (l<heapSize && heap[l].val > heap[largest].val)
	largest = l;
    if (r<heapSize && heap[r].val > heap[largest].val)
	largest = r;

    if (largest != i) {
	heapSwap(i, largest);
	downheap(largest);
    }
}



/*  heapInsert(t, v) -- adds triangle t with value v to the heap  */

void heapInsert(TriIndex t, int v)
{

    heap[heapSize].t = t;
    heap[heapSize].val = v;
    upheap(heapSize++);
}


/*  heapExtractMax() -- remove and return the top triangle from the heap  */

TriIndex heapExtractMax(void)
{
    TriIndex t;

    if (heapSize < 1)
	return 0;

    t = heap[0].t;
    heapSwap(0, --heapSize);
    downheap(0);

    return t;
}



/*  heapEmpty() -- clear the heap  */

void heapEmpty(void)
{
    heapSize = 0;
}

int heapTopVal(void)
{
    if (heapSize > 0) {
	return heap[0].val;
    } else {
	return -1;
    }
}
