/*  split.c -- triangle splitting operations  */

#include "topo.h"



/* Triangle t is split into lt and rt.  Neighbors 1 & 2 of lt and
 * rt are known to be the same size as lt and rt.  Neighbor 3 of
 * lt is neighbor 2 of t (n3lt == n2t).  If n2t is smaller than
 * t then n3lt is same size as lt, o.w. n3lt is bigger than lt (and
 * n1n2t == lt is smaller than n2t == n3lt).
 * Similar gibberish holds for rt.
 */

static void kidsOnSurf(Hier *hp, TriIndex t)
/* Push the surface down to the children of t.  t must not be a leaf. */
{
    TriIndex lt, rt, n2t, n1t;

    assert(ONSURFACE(hp, t));
    assert(!ISLEAF(hp, t));

    hp->node[t].info &= ~SURFACE;
    lt = LCHILD(t);
    PUTONSURFACE(hp, lt);
    rt = RCHILD(t);
    PUTONSURFACE(hp, rt);

    if ((n2t = nbr2(hp, t)) != 0) {
	if (SMALLN2(hp, t)) {
	    hp->node[n2t].info &= ~BIGGER3;
	} else {
	    hp->node[n2t].info |= SMALLER1;
	    hp->node[lt].info |= BIGGER3;
	}
    }
    if ((n1t = nbr1(hp, t)) != 0) {
	if (SMALLN1(hp, t)) {
	    hp->node[n1t].info &= ~BIGGER3;
	} else {
	    hp->node[n1t].info |= SMALLER2;
	    hp->node[rt].info |= BIGGER3;
	}
    }
    if (hp->node[t].info & ALLINFOV) {
	hp->node[lt].info |= ALLINFOV;
	hp->node[rt].info |= ALLINFOV;
    } else if (hp->node[t].info & PARTINFOV) {
	hp->node[lt].info |= triInFOV(lt);
	hp->node[rt].info |= triInFOV(rt);
    }

    if (surfMethod == HEAP) {
	heapInsert(lt, heapBadness(hp, lt));
	heapInsert(rt, heapBadness(hp, rt));
    }
    hp->numSurfTri += 1;
}



void split(Hier *hp, TriIndex t)
{
    TriIndex n3t;

    assert(ONSURFACE(hp, t));

    n3t = nbr3(hp, t);

    if (n3t && BIGN3(hp, t)) {

	split(hp, n3t);			/* propagate split */

	if (t & 1) {
	    n3t = RCHILD(n3t);
	} else {
	    n3t = LCHILD(n3t);
	}

    }

    if (n3t) {				/* t is not LEAF so n3t is not LEAF */
	kidsOnSurf(hp, n3t);
    }

    kidsOnSurf(hp, t);
}
