/*  tri.c -- initialize triangle hierarchy  */

#include <limits.h>
#include "topo.h"

static float X_PER_Y, Y_PER_X;



static int calcErr(Coord2d p1, Coord2d p2, Coord2d p3)
{
    int x1, x2, x3, y1, y2, y3;
    float z1, z2, z3;
    float ystart, yend, xstart, xend;
    float deltaxstart, deltaxend, deltaystart, deltayend;
    int xmini, xmaxi, ymini, ymaxi;
    float z, zstart, zend, deltazstart, deltazend, deltaz;
    float max_eps, eps;
    int x, y;
    int xstarti, xendi, ystarti, yendi;

    x1 = p1[0];
    y1 = p1[1];
    z1 = vscale * dmRef(grid, x1, y1);
    x2 = p2[0];
    y2 = p2[1];
    z2 = vscale * dmRef(grid, x2, y2);
    x3 = p3[0];
    y3 = p3[1];
    z3 = vscale * dmRef(grid, x3, y3);

    assert(x1 < XMAX);
    assert(x2 < XMAX);
    assert(x3 < XMAX);
    assert(y1 < YMAX);
    assert(y2 < YMAX);
    assert(y3 < YMAX);

    max_eps = 0.0;

    if (y1 == y2) {
	if (x1 > x2) {
	    xmini = x2;
	    xmaxi = x1;
	    ymini = y3;
	    ymaxi = y1;

	    xstart = x3;
	    xend = xstart;
	    deltaxstart = -X_PER_Y;
	    deltaxend = X_PER_Y;
	    zstart = z3;
	    zend = z3;
	    deltazstart = (z2 - zstart) / (float)(y1 - y3);
	    deltazend = (z1 - zend) / (float)(y1 - y3);
	} else {
	    xmini = x1;
	    xmaxi = x2;
	    ymini = y1;
	    ymaxi = y3;

	    xstart = x1;
	    xend = x2;
	    deltaxstart = X_PER_Y;
	    deltaxend = -X_PER_Y;
	    zstart = z1;
	    zend = z2;
	    deltazstart = (z3 - zstart) / (float)(y3 - y1);
	    deltazend = (z3 - zend) / (float)(y3 - y1);
	}

	for (y = ymini; y <= ymaxi; ++y) {
	    xstarti = MAX(xmini, ROUND(xstart));
	    xendi = MIN(xmaxi, ROUND(xend));
	    z = zstart;
	    if (xstarti != xendi) {
		deltaz = (zend - zstart) / (xendi - xstarti);
	    } else {
		deltaz = 0.0;
	    }
	    for (x = xstarti; x <= xendi; ++x) {
		eps = ABS(vscale * dmRef(grid, x, y) - z);
		if (eps > max_eps) {
		    max_eps = eps;
		}
		z = z + deltaz;
	    }
	    zstart = zstart + deltazstart;
	    zend = zend + deltazend;
	    xstart = xstart + deltaxstart;
	    xend = xend + deltaxend;
	}
    } else {

	if (x1 == x2) {
	    if (y1 > y2) {
		xmini = x1;
		xmaxi = x3;
		ymini = y2;
		ymaxi = y1;

		ystart = y2;
		yend = y1;
		deltaystart = Y_PER_X;
		deltayend = -Y_PER_X;
		zstart = z2;
		zend = z1;
		deltazstart = (z3 - zstart) / (float)(x3 - x1);
		deltazend = (z3 - zend) / (float)(x3 - x1);
	    } else {
		xmini = x3;
		xmaxi = x1;
		ymini = y1;
		ymaxi = y2;

		ystart = y3;
		yend = ystart;
		deltaystart = -Y_PER_X;
		deltayend = Y_PER_X;
		zstart = z3;
		zend = z3;
		deltazstart = (z1 - zstart) / (float)(x1 - x3);
		deltazend = (z2 - zend) / (float)(x1 - x3);
	    }

	} else {
	    if (x1 < x2) {
		if (y1 > y2) {
		    xmini = x1;
		    xmaxi = x2;
		    ymini = y2;
		    ymaxi = y3;

		    ystart = y1;
		    yend = ystart;
		    deltaystart = -Y_PER_X;
		    deltayend = 0.0;
		    zstart = z1;
		    zend = z1;
		    deltazstart = (z2 - zstart) / (float)(x3 - x1);
		    deltazend = (z3 - zend) / (float)(x3 - x1);
		} else {
		    xmini = x1;
		    xmaxi = x2;
		    ymini = y1;
		    ymaxi = y3;

		    ystart = y1;
		    yend = y3;
		    deltaystart = Y_PER_X;
		    deltayend = 0.0;
		    zstart = z1;
		    zend = z3;
		    deltazstart = (z2 - zstart) / (float)(x2 - x3);
		    deltazend = (z2 - zend) / (float)(x2 - x3);
		}
	    } else {
		if (y1 > y2) {
		    xmini = x2;
		    xmaxi = x3;
		    ymini = y3;
		    ymaxi = y1;

		    ystart = y2;
		    yend = ystart;
		    deltaystart = 0.0;
		    deltayend = Y_PER_X;
		    zstart = z2;
		    zend = z2;
		    deltazstart = (z3 - zstart) / (float)(x3 - x2);
		    deltazend = (z1 - zend) / (float)(x3 - x2);
		} else {
		    xmini = x3;
		    xmaxi = x1;
		    ymini = y3;
		    ymaxi = y2;

		    ystart = y3;
		    yend = y2;
		    deltaystart = 0.0;
		    deltayend = -Y_PER_X;
		    zstart = z3;
		    zend = z2;
		    deltazstart = (z1 - zstart) / (float)(x1 - x3);
		    deltazend = (z1 - zend) / (float)(x1 - x3);
		}
	    }
	}
	for (x = xmini; x <= xmaxi; ++x) {
	    ystarti = MAX(ymini, ROUND(ystart));
	    yendi = MIN(ymaxi, ROUND(yend));
	    z = zstart;
	    if (ystarti != yendi) {
		deltaz = (zend - zstart) / (yendi - ystarti);
	    } else {
		deltaz = 0.0;
	    }
	    for (y = ystarti; y <= yendi; ++y) {
		eps = ABS(vscale * dmRef(grid, x, y) - z);
		if (eps > max_eps) {
		    max_eps = eps;
		}
		z = z + deltaz;
	    }
	    zstart = zstart + deltazstart;
	    zend = zend + deltazend;
	    ystart = ystart + deltaystart;
	    yend = yend + deltayend;
	}
    }

    return (int)(max_eps / errScale + 0.5);
}


/*  midpoint(p1, p2, m) -- set m to midpoint of segment between p1 and p2  */

void midpoint(Coord2d p1, Coord2d p2, Coord2d m)
{
    m[0] = (p1[0] + p2[0]) / 2;
    m[1] = (p1[1] + p2[1]) / 2;
    /*  m[2] = vscale * dmRef(grid, m[0], m[1]); */
}



/* setCoord(p, q) -- copy Coord2d q into Coord2d p */

static void setCoord(Coord2d p, Coord2d q)
{
    p[0] = q[0];
    p[1] = q[1];
}



static void getV(int p, Coord2d p1, Coord2d p2, Coord2d p3)
/*
 * Note: p is TriIndex + 1 so that the bit sequence describes a path in the
 *       tree.
 *       Don't use getV.  Use getVerts.
 */
{
    switch(p) {
    case 0:
    case 1:
    case 2:
	setCoord(p1, se);
	setCoord(p2, nw);
	setCoord(p3, sw);
	return;
    case 3:
	setCoord(p1, nw);
	setCoord(p2, se);
	setCoord(p3, ne);
	return;
    default:
	if (p & 1) {
	    getV(p>>1, p3, p1, p2);
	    midpoint(p1, p3, p3);
	    return;
	} else {
	    getV(p>>1, p2, p3, p1);
	    midpoint(p2, p3, p3);
	    return;
	}
    }
}



void getVerts(TriIndex t, Coord2d p1, Coord2d p2, Coord2d p3)
{
    getV(t + 1, p1, p2, p3);
}



static void printL(int p)
/*
 * Note: p is TriIndex + 1 so that the bit sequence describes a path in the
 *       tree.
 *       Don't use printL.  Use printLabel.
 */
{
    while (p > 1) {
	printL(p>>1);
	if (p & 1) {
	    printf("1");
	} else {
	    printf("0");
	}
    }
}



static void printLabel(TriIndex t)
{
    printL(t + 1);
    printf("\n");
}



static int ceilingLog2(int x)
{
    int m = 1;
    int i = 0;

    while (m < x && m < INT_MAX) {
	m <<= 1;
	++i;
    }
    return i;
}



static int next2Power(int x)
{
    int m = 1;
    while (m < x && m < INT_MAX)
	m <<= 1;
    return m;
}



static void makeDFS(Hier *hp, TriIndex t, Coord2d p1, Coord2d p2, Coord2d p3)
{
    Coord2d mid;
    int r;

    r = inROD(p1, p2, p3);
    hp->node[t].info = r;
    if (ABS(p1[0] - p2[0]) == 1 || ABS(p1[1] - p2[1]) == 1) {
	/* leaf node */
	hp->node[t].err = 0;
    } else {
	if (r == 0) {
	    hp->node[t].err = 0;
	} else if (r == PARTINROD) {
	    hp->node[t].err = MAXERR;
	} else {
	    hp->node[t].err = calcErr(p1, p2, p3);
	}
	midpoint(p1, p2, mid);
	makeDFS(hp, LCHILD(t), p3, p1, mid);
	makeDFS(hp, RCHILD(t), p2, p3, mid);
    }
}



/*  initHier() -- initialize triangle hierarchy  */

Hier *initHier(void)
{
    Hier *hp;
    int dim;
    signed short *p = grid->data;
    int length;
    int z;
    int i;

    XMAX = grid->width;
    YMAX = grid->height;

    X_PER_Y = (float)XMAX / (float)YMAX;
    Y_PER_X = (float)YMAX / (float)XMAX;

    /* find minimum and maximum values, ignoring zeroes and invalid entries */
    EMAX = EMIN = dmRef(grid, grid->width / 2, grid->height / 2);
    length = XMAX * YMAX;
    for(i = 0; i < length; ++i) {
	z = (int)(p[i]);
	if (z == 0 || z < MIN_LEVEL) {
	    continue;
	} else if (z < EMIN) {
	    EMIN = z;
	} else if (z > EMAX) {
	    EMAX = z;
	}
    }

    /* change zero entries to minimum nonzero value seen */
    for(i = 0; i < length; ++i) {
	if (p[i] == 0 || p[i] < MIN_LEVEL)
	    p[i] = EMIN;
    }

    ZMIN = vscale * EMIN;
    ZMAX = vscale * EMAX;
    if (ZMAX == ZMIN)
	errScale = 1.0;
    else
	errScale = (ZMAX - ZMIN) / (float)MAXERR;

    fprintf(stderr,
	"grid size %d x %d, dimensions %d x %d km, elevations %d to %d m\n",
	XMAX, YMAX, (int)(xmeters / 1000 + 0.5), (int)(ymeters / 1000 + 0.5),
	EMIN, EMAX);

    hp = malloc(sizeof(Hier));
    if (hp == NULL) {
	fprintf(stderr, "Can't allocate space for hierarchy\n");
	exit(1);
    }
    dim = next2Power(MAX(XMAX, YMAX) - 1) + 1;
    hp->size = 4 * (dim - 1) * (dim - 1) - 1;
    hp->depth = 2 * ceilingLog2(dim - 1) + 2;
    hp->numSurfTri = 0;

    hp->node = malloc(hp->size * sizeof(Node));
    if (hp->node == NULL) {
	fprintf(stderr, "Can't allocate space for %d nodes\n", hp->size);
	exit(1);
    }

    fprintf(stderr, "Hierarchy uses %d bytes\n",
	    hp->size*sizeof(Node) + sizeof(Hier));

    sw[0] = 0;
    sw[1] = 0;

    se[0] = dim - 1;
    se[1] = 0;

    ne[0] = dim - 1;
    ne[1] = dim - 1;

    nw[0] = 0;
    nw[1] = dim - 1;

    DIMENSION = dim;

    makeDFS(hp, LCHILD(0), se, nw, sw);
    makeDFS(hp, RCHILD(0), nw, se, ne);
    hp->node[0].err = MAXERR;
    hp->node[0].info = 0;

    return hp;
}
